"use strict";
/**
 * Copyright 2023 actions-toolkit authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Install = void 0;
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const core = __importStar(require("@actions/core"));
const httpm = __importStar(require("@actions/http-client"));
const tc = __importStar(require("@actions/tool-cache"));
const semver = __importStar(require("semver"));
const util = __importStar(require("util"));
class Install {
    constructor(opts) {
        this.opts = opts || {};
    }
    install(version, dest) {
        return __awaiter(this, void 0, void 0, function* () {
            const release = yield Install.getRelease(version);
            const fversion = release.tag_name.replace(/^v+|v+$/g, '');
            let toolPath;
            toolPath = tc.find('buildx', fversion, this.platform());
            if (!toolPath) {
                const c = semver.clean(fversion) || '';
                if (!semver.valid(c)) {
                    throw new Error(`Invalid Buildx version "${fversion}".`);
                }
                toolPath = yield this.download(fversion);
            }
            if (this.opts.standalone) {
                return this.setStandalone(toolPath, dest);
            }
            return this.setPlugin(toolPath, dest);
        });
    }
    setStandalone(toolPath, dest) {
        return __awaiter(this, void 0, void 0, function* () {
            const toolBinPath = path_1.default.join(toolPath, os_1.default.platform() == 'win32' ? 'docker-buildx.exe' : 'docker-buildx');
            const binDir = path_1.default.join(dest, 'bin');
            if (!fs_1.default.existsSync(binDir)) {
                fs_1.default.mkdirSync(binDir, { recursive: true });
            }
            const filename = os_1.default.platform() == 'win32' ? 'buildx.exe' : 'buildx';
            const buildxPath = path_1.default.join(binDir, filename);
            fs_1.default.copyFileSync(toolBinPath, buildxPath);
            fs_1.default.chmodSync(buildxPath, '0755');
            core.addPath(binDir);
            return buildxPath;
        });
    }
    setPlugin(toolPath, dest) {
        return __awaiter(this, void 0, void 0, function* () {
            const toolBinPath = path_1.default.join(toolPath, os_1.default.platform() == 'win32' ? 'docker-buildx.exe' : 'docker-buildx');
            const pluginsDir = path_1.default.join(dest, 'cli-plugins');
            if (!fs_1.default.existsSync(pluginsDir)) {
                fs_1.default.mkdirSync(pluginsDir, { recursive: true });
            }
            const filename = os_1.default.platform() == 'win32' ? 'docker-buildx.exe' : 'docker-buildx';
            const pluginPath = path_1.default.join(pluginsDir, filename);
            fs_1.default.copyFileSync(toolBinPath, pluginPath);
            fs_1.default.chmodSync(pluginPath, '0755');
            return pluginPath;
        });
    }
    download(version) {
        return __awaiter(this, void 0, void 0, function* () {
            const targetFile = os_1.default.platform() == 'win32' ? 'docker-buildx.exe' : 'docker-buildx';
            const downloadURL = util.format('https://github.com/docker/buildx/releases/download/v%s/%s', version, this.filename(version));
            const downloadPath = yield tc.downloadTool(downloadURL);
            core.debug(`downloadURL: ${downloadURL}`);
            core.debug(`downloadPath: ${downloadPath}`);
            return yield tc.cacheFile(downloadPath, targetFile, 'buildx', version);
        });
    }
    platform() {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const arm_version = process.config.variables.arm_version;
        return `${os_1.default.platform()}-${os_1.default.arch()}${arm_version ? 'v' + arm_version : ''}`;
    }
    filename(version) {
        let arch;
        switch (os_1.default.arch()) {
            case 'x64': {
                arch = 'amd64';
                break;
            }
            case 'ppc64': {
                arch = 'ppc64le';
                break;
            }
            case 'arm': {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                const arm_version = process.config.variables.arm_version;
                arch = arm_version ? 'arm-v' + arm_version : 'arm';
                break;
            }
            default: {
                arch = os_1.default.arch();
                break;
            }
        }
        const platform = os_1.default.platform() == 'win32' ? 'windows' : os_1.default.platform();
        const ext = os_1.default.platform() == 'win32' ? '.exe' : '';
        return util.format('buildx-v%s.%s-%s%s', version, platform, arch, ext);
    }
    static getRelease(version) {
        return __awaiter(this, void 0, void 0, function* () {
            // FIXME: Use https://raw.githubusercontent.com/docker/actions-toolkit/main/.github/buildx-releases.json when repo public
            const url = `https://raw.githubusercontent.com/docker/buildx/master/.github/releases.json`;
            const http = new httpm.HttpClient('docker-actions-toolkit');
            const resp = yield http.get(url);
            const body = yield resp.readBody();
            const statusCode = resp.message.statusCode || 500;
            if (statusCode >= 400) {
                throw new Error(`Failed to get Buildx release ${version} from ${url} with status code ${statusCode}: ${body}`);
            }
            const releases = JSON.parse(body);
            if (!releases[version]) {
                throw new Error(`Cannot find Buildx release ${version} in ${url}`);
            }
            return releases[version];
        });
    }
}
exports.Install = Install;
//# sourceMappingURL=install.js.map